#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4715 $
$LastChangedDate:: 2021-09-01 15:44:32 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import datetime
import lxml.etree
from symspace.osutils.pathutils import Paths


class JUnitReport:
    """Class to report the build process of components"""

    def __init__(self, name: str, junit_xml_file: str, txt_file: str):
        """Initialize junit report class

        Parameters
        ----------
        name : str
            Name of the testsuite
        junit_xml_file : str
            Path to the junit file
        txt_file : str
            Path to the txt file
        """
        self.name = name
        self.classname = name
        self.tests = 0
        self.errors = 0
        self.time = 0.0
        self.testcases = []
        self.junit_xml_file = junit_xml_file
        self.txt_file = txt_file
        self.t1 = None
        self.t2 = None

    def startTimer(self):
        """start the time for the overall execution time measurement"""
        self.t1 = datetime.datetime.now()

    def addTestCase(self, file: str, elapsed_time: float, msg: str) -> bool:
        """
        Add a testcase to the testsuite

        Parameters
        ----------
        file : str
            Path to the component
        elapsed_time: float
            Time to build the componente
        msg: str
            Build return message
        Returns
        -------
        failed: bool
            True if the build resulted in an error
        """
        if self.t1 is None:
            print('Starting Timer to late!')
            self.startTimer()

        if 'error' in msg.lower():
            error_msg = msg
        else:
            error_msg = ''

        file_id = file.replace(Paths.getComponentSpaceRoot(), '.')

        testcase = dict(file=file_id, time=elapsed_time, error_msg=error_msg)
        self.testcases.append(testcase)
        self.tests += 1
        if self.testFailed(error_msg):
            failed = True
            self.errors += 1
        else:
            failed = False

        '''increase overall test time'''
        self.time += elapsed_time
        self.t2 = datetime.datetime.now()

        return failed

    @staticmethod
    def testFailed(error_msg) -> bool:
        """
        Check if the test failed

        Parameters
        ----------
        error_msg : str
            Error message

        Returns
        -------
        bool
        """
        if error_msg != '':
            return True
        else:
            return False

    def writeXML(self):
        """
        Write a junit xml file from the collected testcases

        """
        jtestsuite = lxml.etree.Element('testsuite',
                                        name=self.name, tests=str(self.tests),
                                        errors=str(self.errors), time='{:.3f}'.format(self.time))

        for testcase in self.testcases:
            file = testcase['file']
            time = '{:.3f}'.format(testcase['time'])
            error_msg = testcase['error_msg']

            jtestcase = lxml.etree.SubElement(jtestsuite, 'testcase',
                                              classname=self.classname,
                                              name='build: {}'.format(file),
                                              time=time)
            if self.testFailed(error_msg):
                '''add an error element'''
                lxml.etree.SubElement(jtestcase, 'error', message='{}'.format(error_msg))

        xtestsuite = lxml.etree.tostring(jtestsuite,
                                         pretty_print=True,
                                         xml_declaration=True,
                                         encoding='UTF-8').decode('ascii')

        with open(self.junit_xml_file, 'w') as fh:
            fh.write(xtestsuite)

    def writeTxt(self):
        """
        Write a text file from the collected testcases

        """
        with open(self.txt_file, 'w') as fh:

            fh.write('Start build: {}\n\n'.format(str(self.t1)))

            for testcase in self.testcases:
                file = testcase['file']
                error_msg = testcase['error_msg']

                if error_msg != '':
                    fh.write('{}\n    {}\n'.format(file, error_msg))
                else:
                    fh.write('{} successfully built\n'.format(file))

            fh.write('\nEnd build: {}'.format(str(self.t2)))

    @classmethod
    def fromXML(cls, junit_xml):
        """
        Create an instance of this class from an xml file

        Parameters
        ----------
        junit_xml : str
            Path to the junit file

        Returns
        -------
        obj : JUnitReport
            JUnitReport instance
        """
        tree = lxml.etree.parse(junit_xml)
        root = tree.getroot()
        name = root.get('name')
        obj = cls(name, junit_xml, txt_file='')
        obj.tests = int(root.get('tests'))
        obj.errors = int(root.get('errors'))
        obj.time = float(root.get('time'))
        obj.testcases = []  # unfinished

        return obj
