# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4484 $
$LastChangedDate:: 2021-06-15 16:57:50 +0200#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import time
import psutil
import subprocess

"""All functions may already exist in the latest SyMSpace version
   This module only provides compatibility for ComponentSpace Users with an older SyMSpace version
"""


# noinspection DuplicatedCode
def countRunningProcesses(processes: list) -> int:
    """
    Count the number of running processes in the input list

    Parameters
    ----------
    processes : list
        List of processes

    Returns
    -------
    n_running: int
        Number of running processes
    """
    n_running = 0
    for proc in processes:
        if proc.poll() is None:
            n_running += 1
    return n_running


# noinspection DuplicatedCode
def waitForProcesses(process_list: list, n_max_proc: int = None, max_cpu_percent: float = 75.0,
                     max_ram_percent: float = 75.0, cpu_measure_interval: float = 2, wait_seconds: float = None) -> int:
    """
    Wait until the number of running processes, CPU or RAM usage is
    Parameters
    ----------
    process_list: list
        List of processes
    n_max_proc: int
        Maximum number of running processes
    max_cpu_percent: float
        Maximum CPU load
    max_ram_percent: float
        Maximum RAM usage
    cpu_measure_interval: float
        CPU measurement interval
    wait_seconds : float
        Additional wait seconds

    Returns
    -------
    n_running: int
        Number of running processes
    """
    if n_max_proc is None:
        n_max_proc = psutil.cpu_count(logical=True)

    cpu = psutil.cpu_percent(cpu_measure_interval)  # measure cpu load in percent for X seconds -> time.sleep(X)
    ram = psutil.virtual_memory()[2]  # get ram usage in percent
    n_running = countRunningProcesses(process_list)

    while n_running > n_max_proc or cpu > max_cpu_percent or ram > max_ram_percent:
        print('Running proceses: {:d}, CPU: {:.0f}%, RAM: {:.0f}%'.format(n_running, cpu, ram))
        print('\t==> {}'.format('waiting to finish some processes'))
        if wait_seconds is not None:
            time.sleep(wait_seconds)
        cpu = psutil.cpu_percent(cpu_measure_interval)  # measure cpu load in percent for X seconds -> time.sleep(X)
        ram = psutil.virtual_memory()[2]  # get ram usage in percent
        n_running = countRunningProcesses(process_list)

    return n_running


# noinspection DuplicatedCode
def startProcess(cmd_args: list, cmd_kwargs: dict = None, cwd: str = None, hide_console: bool = True,
                 stdout=None, stderr=None,
                 wait_to_start: bool = True, wait_to_finish: bool = False) -> subprocess.Popen:
    """
    Run an external process

    Parameters
    ----------
    cmd_args: list
        list of strings, with additional commandline arguments
    cmd_kwargs: dict
        dict with additional commandline arguments, key and value have to be of type string!
    cwd : str
        Current working directory
    hide_console : bool
        If True the subprocess console is during the build process is hidden / does not pop up, Default: True
    stdout :
        Output stream, subprocess.PIPE (Default) or None
    stderr :
        Error stream, subprocess.PIPE (Default) or None

    wait_to_start: bool
        Wait until process has started
    wait_to_finish : bool
        Wait until process has finished

    Returns
    -------
    subprocess.Popen
    """
    si = subprocess.STARTUPINFO()
    if hide_console:
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW

    run_args = []
    if cmd_args is not None:
        for value in cmd_args:
            if isinstance(value, (int, float)):
                '''convert numeric value to string'''
                value = str(value)
            run_args.append(value)
    if cmd_kwargs is not None:
        for key, value in cmd_kwargs.items():
            if isinstance(value, (int, float)):
                '''convert numeric value to string'''
                value = str(value)
            run_args.append(key)
            run_args.append(value)

    try:
        process = subprocess.Popen(run_args, cwd=cwd, stdout=stdout, stderr=stderr, startupinfo=si)
        if wait_to_start:
            '''problably unnecessary -> but just to make sure that the process is really alive'''
            while process.poll() is not None:
                print('Waiting for process to start')
                time.sleep(1)
            # print('Process has started')
        if wait_to_finish:
            # print('Waiting for process to finish')
            process.wait()
        return process
    except subprocess.CalledProcessError as ex:
        if ex.stderr is None:
            raise Exception('Process failed with exit status {}'.format(ex.returncode))
        else:
            raise Exception(ex.stderr.decode('utf-8'))
    except subprocess.TimeoutExpired as ex:
        raise TimeoutError('Timeout after {} s'.format(ex.timeout))
